@@ -18,6 +18,7 @@
 
 import pyghmi.exceptions as exc
 import struct
+import threading
 
 from pyghmi.ipmi.private import constants
 from pyghmi.ipmi.private import session
@@ -38,10 +39,11 @@ class Console(object):
     :param kg: optional parameter for BMCs configured to require it
     """
 
-    #TODO(jbjohnso): still need an exit and a data callin function
+    # TODO(jbjohnso): still need an exit and a data callin function
     def __init__(self, bmc, userid, password,
                  iohandler, port=623,
                  force=False, kg=None):
+        self.outputlock = threading.RLock()
         self.keepaliveid = None
         self.connected = False
         self.broken = False
@@ -54,6 +56,7 @@ class Console(object):
         self.awaitingack = False
         self.activated = False
         self.force_session = force
+        self.port = port
         self.ipmi_session = session.Session(bmc=bmc,
                                             userid=userid,
                                             password=password,
@@ -70,19 +73,19 @@ class Console(object):
         if 'error' in response:
             self._print_error(response['error'])
             return
-        #Send activate sol payload directive
-        #netfn= 6 (application)
-        #command = 0x48 (activate payload)
-        #data = (1, sol payload type
+        # Send activate sol payload directive
+        # netfn= 6 (application)
+        # command = 0x48 (activate payload)
+        # data = (1, sol payload type
         #        1, first instance
         #        0b11000000, -encrypt, authenticate,
         #                      disable serial/modem alerts, CTS fine
         #        0, 0, 0 reserved
         response = self.ipmi_session.raw_command(netfn=0x6, command=0x48,
                                                  data=(1, 1, 192, 0, 0, 0))
-        #given that these are specific to the command,
-        #it's probably best if one can grep the error
-        #here instead of in constants
+        # given that these are specific to the command,
+        # it's probably best if one can grep the error
+        # here instead of in constants
         sol_activate_codes = {
             0x81: 'SOL is disabled',
             0x82: 'Maximum SOL session count reached',
@@ -118,31 +121,37 @@ class Console(object):
             self._print_error(response['error'])
             return
         self.activated = True
-        #data[0:3] is reserved except for the test mode, which we don't use
+        # data[0:3] is reserved except for the test mode, which we don't use
         data = response['data']
         self.maxoutcount = (data[5] << 8) + data[4]
-           #BMC tells us this is the maximum allowed size
-        #data[6:7] is the promise of how small packets are going to be, but we
-        #don't have any reason to worry about it
-        if (data[8] + (data[9] << 8)) != 623:
-            #TODO(jbjohnso): support atypical SOL port number
+        # BMC tells us this is the maximum allowed size
+        # data[6:7] is the promise of how small packets are going to be, but we
+        # don't have any reason to worry about it
+        # some BMCs disagree on the endianness, so do both
+        valid_ports = (self.port, struct.unpack(
+            '<H', struct.pack('>H', self.port))[0])
+        if (data[8] + (data[9] << 8)) not in valid_ports:
+            # TODO(jbjohnso): support atypical SOL port number
             raise NotImplementedError("Non-standard SOL Port Number")
-        #ignore data[10:11] for now, the vlan detail, shouldn't matter to this
-        #code anyway...
-        #NOTE(jbjohnso):
-        #We will use a special purpose keepalive
+        # ignore data[10:11] for now, the vlan detail, shouldn't matter to this
+        # code anyway...
+        # NOTE(jbjohnso):
+        # We will use a special purpose keepalive
+        if self.ipmi_session.sol_handler is not None:
+            # If there is erroneously another SOL handler already, notify
+            # it of newly established session
+            self.ipmi_session.sol_handler({'error': 'Session Disconnected'})
         self.keepaliveid = self.ipmi_session.register_keepalive(
             cmd={'netfn': 6, 'command': 0x4b, 'data': (1, 1)},
             callback=self._got_payload_instance_info)
         self.ipmi_session.sol_handler = self._got_sol_payload
         self.connected = True
-        if len(self.pendingoutput) > 0:
-            self._sendpendingoutput()
+        # self._sendpendingoutput() checks len(self._sendpendingoutput)
+        self._sendpendingoutput()
 
     def _got_payload_instance_info(self, response):
         if 'error' in response:
             self.activated = False
-            self.ipmi_session.unregister_keepalive(self.keepaliveid)
             self._print_error(response['error'])
             return
         currowner = struct.unpack(
@@ -150,7 +159,6 @@ class Console(object):
         if currowner[0] != self.ipmi_session.sessionid:
             # the session is deactivated or active for something else
             self.activated = False
-            self.ipmi_session.unregister_keepalive(self.keepaliveid)
             self._print_error('SOL deactivated')
             return
         # ok, still here, that means session is alive, but another
@@ -161,14 +169,15 @@ class Console(object):
         # correlates at all to an ipmi channel to check mux
 
     def _addpendingdata(self, data):
-        if isinstance(data, dict):
-            self.pendingoutput.append(data)
-        else:  # it is a text situation
-            if (len(self.pendingoutput) == 0 or
-                    isinstance(self.pendingoutput[-1], dict)):
+        with self.outputlock:
+            if isinstance(data, dict):
                 self.pendingoutput.append(data)
-            else:
-                self.pendingoutput[-1] += data
+            else:  # it is a text situation
+                if (len(self.pendingoutput) == 0 or
+                        isinstance(self.pendingoutput[-1], dict)):
+                    self.pendingoutput.append(data)
+                else:
+                    self.pendingoutput[-1] += data
 
     def _got_cons_input(self, handle):
         """Callback for handle events detected by ipmi session
@@ -180,7 +189,8 @@ class Console(object):
     def close(self):
         """Shut down an SOL session,
         """
-        self.ipmi_session.unregister_keepalive(self.keepaliveid)
+        if self.ipmi_session:
+            self.ipmi_session.unregister_keepalive(self.keepaliveid)
         if self.activated:
             try:
                 self.ipmi_session.raw_command(netfn=6, command=0x49,
@@ -217,20 +227,24 @@ class Console(object):
         return session.Session.wait_for_rsp(timeout=timeout)
 
     def _sendpendingoutput(self):
-        if isinstance(self.pendingoutput[0], dict):
-            if 'break' in self.pendingoutput[0]:
-                self._sendoutput("", sendbreak=True)
+        with self.outputlock:
+            if len(self.pendingoutput) == 0:
+                return
+            if isinstance(self.pendingoutput[0], dict):
+                if 'break' in self.pendingoutput[0]:
+                    self._sendoutput("", sendbreak=True)
+                else:
+                    raise ValueError
+                del self.pendingoutput[0]
+                return
+            if len(self.pendingoutput[0]) > self.maxoutcount:
+                chunk = self.pendingoutput[0][:self.maxoutcount]
+                self.pendingoutput[0] = self.pendingoutput[0][
+                    self.maxoutcount:]
             else:
-                raise ValueError
-            del self.pendingoutput[0]
-            return
-        if len(self.pendingoutput[0]) > self.maxoutcount:
-            chunk = self.pendingoutput[0][:self.maxoutcount]
-            self.pendingoutput[0] = self.pendingoutput[0][self.maxoutcount:]
-        else:
-            chunk = self.pendingoutput[0]
-            del self.pendingoutput[0]
-        self._sendoutput(chunk)
+                chunk = self.pendingoutput[0]
+                del self.pendingoutput[0]
+            self._sendoutput(chunk)
 
     def _sendoutput(self, output, sendbreak=False):
         self.myseq += 1
@@ -270,6 +284,12 @@ class Console(object):
 
     def _print_error(self, error):
         self.broken = True
+        if self.ipmi_session:
+            self.ipmi_session.unregister_keepalive(self.keepaliveid)
+            if (self.ipmi_session.sol_handler and
+                    self.ipmi_session.sol_handler.__self__ is self):
+                self.ipmi_session.sol_handler = None
+            self.ipmi_session = None
         if type(error) == dict:
             self._print_data(error)
         else:
@@ -287,9 +307,9 @@ class Console(object):
     def _got_sol_payload(self, payload):
         """SOL payload callback
         """
-        #TODO(jbjohnso) test cases to throw some likely scenarios at functions
-        #for example, retry with new data, retry with no new data
-        #retry with unexpected sequence number
+        # TODO(jbjohnso) test cases to throw some likely scenarios at functions
+        # for example, retry with new data, retry with no new data
+        # retry with unexpected sequence number
         if type(payload) == dict:  # we received an error condition
             self.activated = False
             self._print_error(payload)
@@ -301,15 +321,16 @@ class Console(object):
         poweredoff = payload[3] & 0b100000
         deactivated = payload[3] & 0b10000
         breakdetected = payload[3] & 0b100
-        #for now, ignore overrun.  I assume partial NACK for this reason or for
-        #no reason would be treated the same, new payload with partial data
+        # for now, ignore overrun.  I assume partial NACK for this reason or
+        # for no reason would be treated the same, new payload with partial
+        # data.
         remdata = ""
         remdatalen = 0
         if newseq != 0:  # this packet at least has some data to send to us..
             if len(payload) > 4:
                 remdatalen = len(payload[4:])  # store remote len before dupe
-                    #retry logic, we must ack *this* many even if it is
-                    #a retry packet with new partial data
+                # retry logic, we must ack *this* many even if it is
+                # a retry packet with new partial data
                 remdata = struct.pack("%dB" % remdatalen, *payload[4:])
             if newseq == self.remseq:  # it is a retry, but could have new data
                 if remdatalen > self.lastsize:
@@ -322,16 +343,16 @@ class Console(object):
             if remdata:  # Do not subject callers to empty data
                 self._print_data(remdata)
             ackpayload = (0, self.remseq, remdatalen, 0)
-            #Why not put pending data into the ack? because it's rare
-            #and might be hard to decide what to do in the context of
-            #retry situation
+            # Why not put pending data into the ack? because it's rare
+            # and might be hard to decide what to do in the context of
+            # retry situation
             try:
                 self.send_payload(ackpayload, retry=False)
             except exc.IpmiException:
-                #if the session is broken, then close the SOL session
+                # if the session is broken, then close the SOL session
                 self.close()
         if self.myseq != 0 and ackseq == self.myseq:  # the bmc has something
-                                                      # to say about last xmit
+            # to say about last xmit
             self.awaitingack = False
             if nacked and not breakdetected:  # the BMC was in some way unhappy
                 if poweredoff:
@@ -343,17 +364,24 @@ class Console(object):
                     # also add pending output for efficiency and ease
                     newtext = self.lastpayload[4 + ackcount:]
                     newtext = struct.pack("B"*len(newtext), *newtext)
-                    if (self.pendingoutput and
-                            not isinstance(self.pendingoutput[0], dict)):
-                        self.pendingoutput[0] = newtext + self.pendingoutput[0]
-                    else:
-                        self.pendingoutput = [newtext] + self.pendingoutput
-                    self._sendpendingoutput()
-            if len(self.pendingoutput) > 0:
-                self._sendpendingoutput()
-        elif self.awaitingack:  # session marked us as happy, but we are not
-                #this does mean that we will occasionally retry a packet
-                #sooner than retry suggests, but that's no big deal
+                    with self.outputlock:
+                        if (self.pendingoutput and
+                                not isinstance(self.pendingoutput[0], dict)):
+                            self.pendingoutput[0] = \
+                                newtext + self.pendingoutput[0]
+                        else:
+                            self.pendingoutput = [newtext] + self.pendingoutput
+            # self._sendpendingoutput() checks len(self._sendpendingoutput)
+            self._sendpendingoutput()
+        elif ackseq != 0 and self.awaitingack:
+            # if an ack packet came in, but did not match what we
+            # expected, retry our payload now.
+            # the situation that was triggered was a senseless retry
+            # when data came in while we xmitted.  In theory, a BMC
+            # should handle a retry correctly, but some do not, so
+            # try to mitigate by avoiding overeager retries
+            # occasional retry of a packet
+            # sooner than timeout suggests is evidently a big deal
             self.send_payload(payload=self.lastpayload)
 
     def main_loop(self):
@@ -364,9 +392,130 @@ class Console(object):
         to provide their own event loop behavior, though this could be used
         within the greenthread implementation of caller's choice if desired.
         """
-        #wait_for_rsp promises to return a false value when no sessions are
-        #alive anymore
-        #TODO(jbjohnso): wait_for_rsp is not returning a true value for our own
-        #session
+        # wait_for_rsp promises to return a false value when no sessions are
+        # alive anymore
+        # TODO(jbjohnso): wait_for_rsp is not returning a true value for our
+        # own session
         while (1):
             session.Session.wait_for_rsp(timeout=600)
+
+
+class ServerConsole(Console):
+    """IPMI SOL class.
+
+    This object represents an SOL channel, multiplexing SOL data with
+    commands issued by ipmi.command.
+
+    :param session: IPMI session
+    :param iohandler: I/O handler
+    """
+
+    def __init__(self, _session, iohandler, force=False):
+        self.outputlock = threading.RLock()
+        self.keepaliveid = None
+        self.connected = True
+        self.broken = False
+        self.out_handler = iohandler
+        self.remseq = 0
+        self.myseq = 0
+        self.lastsize = 0
+        self.retriedpayload = 0
+        self.pendingoutput = []
+        self.awaitingack = False
+        self.activated = True
+        self.force_session = force
+        self.ipmi_session = _session
+        self.ipmi_session.sol_handler = self._got_sol_payload
+        self.maxoutcount = 256
+        self.poweredon = True
+
+        session.Session.wait_for_rsp(0)
+
+    def _got_sol_payload(self, payload):
+        """SOL payload callback
+        """
+        # TODO(jbjohnso) test cases to throw some likely scenarios at functions
+        # for example, retry with new data, retry with no new data
+        # retry with unexpected sequence number
+        if type(payload) == dict:  # we received an error condition
+            self.activated = False
+            self._print_error(payload)
+            return
+        newseq = payload[0] & 0b1111
+        ackseq = payload[1] & 0b1111
+        ackcount = payload[2]
+        nacked = payload[3] & 0b1000000
+        breakdetected = payload[3] & 0b10000
+        # for now, ignore overrun.  I assume partial NACK for this reason or
+        # for no reason would be treated the same, new payload with partial
+        # data.
+        remdata = ""
+        remdatalen = 0
+        flag = 0
+        if not self.poweredon:
+            flag |= 0b1100000
+        if not self.activated:
+            flag |= 0b1010000
+        if newseq != 0:  # this packet at least has some data to send to us..
+            if len(payload) > 4:
+                remdatalen = len(payload[4:])  # store remote len before dupe
+                # retry logic, we must ack *this* many even if it is
+                # a retry packet with new partial data
+                remdata = struct.pack("%dB" % remdatalen, *payload[4:])
+            if newseq == self.remseq:  # it is a retry, but could have new data
+                if remdatalen > self.lastsize:
+                    remdata = remdata[4 + self.lastsize:]
+                else:  # no new data...
+                    remdata = ""
+            else:  # TODO(jbjohnso) what if remote sequence number is wrong??
+                self.remseq = newseq
+            self.lastsize = remdatalen
+            ackpayload = (0, self.remseq, remdatalen, flag)
+            # Why not put pending data into the ack? because it's rare
+            # and might be hard to decide what to do in the context of
+            # retry situation
+            try:
+                self.send_payload(ackpayload, retry=False)
+            except exc.IpmiException:
+                # if the session is broken, then close the SOL session
+                self.close()
+            if remdata:  # Do not subject callers to empty data
+                self._print_data(remdata)
+        if self.myseq != 0 and ackseq == self.myseq:  # the bmc has something
+            # to say about last xmit
+            self.awaitingack = False
+            if nacked and not breakdetected:  # the BMC was in some way unhappy
+                newtext = self.lastpayload[4 + ackcount:]
+                newtext = struct.pack("B"*len(newtext), *newtext)
+                with self.outputlock:
+                    if (self.pendingoutput and
+                            not isinstance(self.pendingoutput[0], dict)):
+                        self.pendingoutput[0] = newtext + self.pendingoutput[0]
+                    else:
+                        self.pendingoutput = [newtext] + self.pendingoutput
+            # self._sendpendingoutput() checks len(self._sendpendingoutput)
+            self._sendpendingoutput()
+        elif ackseq != 0 and self.awaitingack:
+            # if an ack packet came in, but did not match what we
+            # expected, retry our payload now.
+            # the situation that was triggered was a senseless retry
+            # when data came in while we xmitted.  In theory, a BMC
+            # should handle a retry correctly, but some do not, so
+            # try to mitigate by avoiding overeager retries
+            # occasional retry of a packet
+            # sooner than timeout suggests is evidently a big deal
+            self.send_payload(payload=self.lastpayload)
+
+    def send_payload(self, payload, payload_type=1, retry=True,
+                     needskeepalive=False):
+        while not (self.connected or self.broken):
+            session.Session.wait_for_rsp(timeout=10)
+        self.ipmi_session.send_payload(payload,
+                                       payload_type=payload_type,
+                                       retry=retry,
+                                       needskeepalive=needskeepalive)
+
+    def close(self):
+        """Shut down an SOL session,
+        """
+        self.activated = False
