@@ -14,11 +14,11 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
+import pyghmi.constants as const
+
 IPMI_BMC_ADDRESS = 0x20
 IPMI_SEND_MESSAGE_CMD = 0x34
 
-import pyghmi.constants as const
-
 
 payload_types = {
     'ipmi': 0x0,
@@ -37,7 +37,7 @@ sensor_type_codes = {
     2: 'Voltage',
     3: 'Current',
     4: 'Fan',
-    5: 'Chassis Intrusion',
+    5: 'Physical Security',
     6: 'Platform Security',
     7: 'Processor',
     8: 'Power Supply',
@@ -238,7 +238,7 @@ generic_type_offsets = {
             'deassertion_severity': const.Health.Warning,
         },
         1: {
-            'desc': 'Perfermance lags',
+            'desc': 'Performance lags',
             'severity': const.Health.Warning,
             'deassertion_desc': 'Performance no longer lags',
             'deassertion_severity': const.Health.Ok,
@@ -394,7 +394,7 @@ generic_type_offsets = {
         1: {
             'desc': 'Not redundant',  # redundancy lost
             'severity': const.Health.Warning,
-            'deassertion_desc': 'Redondancy restored',
+            'deassertion_desc': 'Redundancy restored',
             'deassertion_severity': const.Health.Ok,
         },
         2: {
@@ -470,33 +470,33 @@ sensor_type_offsets = {
     # observed
     5: {
         0: {
-            'desc': 'General',
+            'desc': 'Chassis intrusion',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Deasserted',
+            'deassertion_desc': 'Chassis intrusion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         1: {
-            'desc': 'Drive Bay',
+            'desc': 'Drive area intrusion',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Drive Bay deasserted',
+            'deassertion_desc': 'Drive area intrusion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         2: {
-            'desc': 'I/O card area',
+            'desc': 'I/O card area intrusion',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'I/O card area deasserted',
+            'deassertion_desc': 'I/O card area intrusion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         3: {
-            'desc': 'Processor area',
+            'desc': 'Processor area intrusion',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Processor area deasserted',
+            'deassertion_desc': 'Processor area intrusion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         4: {
-            'desc': 'Lost LAN',
+            'desc': 'LAN connection lost',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'LAN restored',
+            'deassertion_desc': 'LAN connection restored',
             'deassertion_severity': const.Health.Ok,
         },
         5: {
@@ -506,9 +506,9 @@ sensor_type_offsets = {
             'deassertion_severity': const.Health.Ok,
         },
         6: {
-            'desc': 'Fan area',
+            'desc': 'Fan area intrusion',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Fan area deasserted',
+            'deassertion_desc': 'Fan area intrusion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
     },
@@ -554,9 +554,9 @@ sensor_type_offsets = {
     },
     7: {
         0: {
-            'desc': 'Internal Error',
+            'desc': 'Internal error',
             'severity': const.Health.Failed,
-            'deassertion_desc': 'Internal Error Recovered',
+            'deassertion_desc': 'Internal error recovered',
             'deassertion_severity': const.Health.Ok,
         },
         1: {
@@ -617,7 +617,7 @@ sensor_type_offsets = {
         0xa: {
             'desc': 'Throttled',
             'severity': const.Health.Warning,
-            'deassertion_desc': 'Unthrottled',
+            'deassertion_desc': 'No longer throttled',
             'deassertion_severity': const.Health.Ok,
         },
         0xb: {
@@ -654,21 +654,21 @@ sensor_type_offsets = {
             'deassertion_severity': const.Health.Ok,
         },
         3: {
-            'desc': 'Input lost',
+            'desc': 'Power input lost',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Input restored',
+            'deassertion_desc': 'Power input restored',
             'deassertion_severity': const.Health.Ok,
         },
         4: {
-            'desc': 'Input out of range or lost',
+            'desc': 'Power input out of range or lost',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Input in range',
+            'deassertion_desc': 'Power input present and in range',
             'deassertion_severity': const.Health.Ok,
         },
         5: {
-            'desc': 'Input out of range',
+            'desc': 'Power input out of range',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Input in range',
+            'deassertion_desc': 'Power input in range',
             'deassertion_severity': const.Health.Ok,
         },
         6: {
@@ -745,7 +745,7 @@ sensor_type_offsets = {
         1: {
             'desc': 'Uncorrectable error',
             'severity': const.Health.Failed,
-            'deassertion_desc': 'Unorrectable error deasserted',
+            'deassertion_desc': 'Uncorrectable error deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         2: {
@@ -829,7 +829,7 @@ sensor_type_offsets = {
             'deassertion_severity': const.Health.Ok,
         },
         4: {
-            'desc': 'Drive consitency check in progress',
+            'desc': 'Drive consistency check in progress',
             'severity': const.Health.Ok,
             'deassertion_desc': 'Drive consistency check complete',
             'deassertion_severity': const.Health.Ok,
@@ -912,7 +912,7 @@ sensor_type_offsets = {
         },
         5: {
             'desc': 'Event log nearly full',
-            'severity': const.Health.Warning,
+            'severity': const.Health.Ok,
             'deassertion_desc': 'Event log no longer nearly full',
             'deassertion_severity': const.Health.Ok,
         },
@@ -1067,7 +1067,7 @@ sensor_type_offsets = {
             'deassertion_severity': const.Health.Ok,
         },
     },
-    0x19: {  # chipset
+    0x19: {  # chip set
         0: {
             'desc': 'Soft power control failure',
             'severity': const.Health.Critical,
@@ -1121,9 +1121,9 @@ sensor_type_offsets = {
             'deassertion_severity': const.Health.Ok,
         },
         4: {
-            'desc': 'Autoboot to diagnostic',
+            'desc': 'Automatic boot to diagnostic',
             'severity': const.Health.Warning,
-            'deassertion_desc': 'Autoboot to diagnostic deasserted',
+            'deassertion_desc': 'Automatic boot to diagnostic deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         5: {
@@ -1194,7 +1194,7 @@ sensor_type_offsets = {
         2: {
             'desc': 'Network boot completed',
             'severity': const.Health.Ok,
-            'deassertion_desc': 'Network boot completetion deasserted',
+            'deassertion_desc': 'Network boot completion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         3: {
@@ -1206,7 +1206,7 @@ sensor_type_offsets = {
         4: {
             'desc': 'CD boot completed',
             'severity': const.Health.Ok,
-            'deassertion_desc': 'CD boot completetion deasserted',
+            'deassertion_desc': 'CD boot completion deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         5: {
@@ -1386,9 +1386,9 @@ sensor_type_offsets = {
             'deassertion_severity': const.Health.Ok,
         },
         0x6: {
-            'desc': 'Hibernated or Off',
+            'desc': 'Hibernated or off',
             'severity': const.Health.Ok,
-            'deassertion_desc': 'Hibernateor off deasserted',
+            'deassertion_desc': 'Hibernate or off deasserted',
             'deassertion_severity': const.Health.Ok,
         },
         0x7: {
@@ -1608,7 +1608,7 @@ sensor_type_offsets = {
         3: {
             'desc': 'Account disabled due to failure count',
             'severity': const.Health.Critical,
-            'deassertion_desc': 'Account reenabled',
+            'deassertion_desc': 'Account re-enabled',
             'deassertion_severity': const.Health.Ok,
         },
     },
@@ -1661,7 +1661,7 @@ sensor_type_offsets = {
             'desc': 'Successful software/firmware change',
             'severity': const.Health.Ok,
             'deassertion_desc':
-            'Succussful software/firmware change deasserted',
+            'Successful software/firmware change deasserted',
             'deassertion_severity': const.Health.Ok,
         },
     },
