@@ -677,16 +677,9 @@ ddr3_bus_width = {
     3: 64,
 }
 
-speed_by_clock = {
-    800: 6400,
-    1066: 8500,
-    1333: 10600,
-    1600: 12800,
-    1867: 14900,
-    2132: 17000,
-    2133: 17000,
-    2134: 17000,
-}
+
+def speed_from_clock(clock):
+    return int(clock * 8 - (clock * 8 % 100))
 
 
 def decode_manufacturer(index, mfg):
@@ -731,7 +724,7 @@ class SPD(object):
         fineoffset = (finetime * fineoffset) * 10**-3
         mtb = spd[10] / float(spd[11])
         clock = 2 // ((mtb * spd[12] + fineoffset)*10**-3)
-        self.info['speed'] = speed_by_clock.get(clock, 'Unknown')
+        self.info['speed'] = speed_from_clock(clock)
         self.info['ecc'] = (spd[8] & 0b11000) != 0
         self.info['module_type'] = module_types.get(spd[3] & 0xf, 'Unknown')
         sdramcap = ddr3_module_capacity[spd[4] & 0xf]
@@ -742,9 +735,10 @@ class SPD(object):
         self.info['manufacturer'] = decode_manufacturer(spd[117], spd[118])
         self.info['manufacture_location'] = spd[119]
         self.info['manufacture_date'] = decode_spd_date(spd[120], spd[121])
-        self.info['serial'] = struct.unpack(
-            '>I', struct.pack('4B', *spd[122:126]))[0]
-        self.info['model'] = struct.pack('18B', *spd[128:146])
+        self.info['serial'] = hex(struct.unpack(
+            '>I', struct.pack('4B', *spd[122:126]))[0])[2:].rjust(8, '0')
+        self.info['model'] = struct.pack('18B', *spd[128:146]).strip(
+            '\x00\xff ')
 
     def _decode_ddr4(self):
         spd = self.rawdata
@@ -753,7 +747,7 @@ class SPD(object):
             if fineoffset & 0b10000000:
                 fineoffset = 0 - ((fineoffset ^ 0xff) + 1)
             clock = 2 // ((0.125 * spd[18] + fineoffset * 0.001) * 0.001)
-            self.info['speed'] = speed_by_clock.get(clock, 'Unknown')
+            self.info['speed'] = speed_from_clock(clock)
         else:
             self.info['speed'] = 'Unknown'
         self.info['ecc'] = (spd[13] & 0b11000) == 0b1000
@@ -767,6 +761,7 @@ class SPD(object):
         self.info['manufacturer'] = decode_manufacturer(spd[320], spd[321])
         self.info['manufacture_location'] = spd[322]
         self.info['manufacture_date'] = decode_spd_date(spd[323], spd[324])
-        self.info['serial'] = struct.unpack(
-            '>I', struct.pack('4B', *spd[325:329]))[0]
-        self.info['model'] = struct.pack('18B', *spd[329:347])
+        self.info['serial'] = hex(struct.unpack(
+            '>I', struct.pack('4B', *spd[325:329]))[0])[2:].rjust(8, '0')
+        self.info['model'] = struct.pack('18B', *spd[329:347]).strip(
+            '\x00\xff ')
