@@ -34,6 +34,7 @@ import pyghmi.constants as const
 import pyghmi.exceptions as exc
 import pyghmi.ipmi.private.constants as ipmiconst
 import struct
+import weakref
 
 TYPE_UNKNOWN = 0
 TYPE_SENSOR = 1
@@ -45,8 +46,8 @@ def ones_complement(value, bits):
     # complement prevalent in ipmi spec
     signbit = 0b1 << (bits - 1)
     if value & signbit:
-        #if negative, subtract 1, then take 1s
-        #complement given bits width
+        # if negative, subtract 1, then take 1s
+        # complement given bits width
         return 0 - (value ^ ((0b1 << bits) - 1))
     else:
         return value
@@ -57,8 +58,8 @@ def twos_complement(value, bits):
     # complement prevalent in ipmi spec
     signbit = 0b1 << (bits - 1)
     if value & signbit:
-        #if negative, subtract 1, then take 1s
-        #complement given bits width
+        # if negative, subtract 1, then take 1s
+        # complement given bits width
         return 0 - ((value - 1) ^ ((0b1 << bits) - 1))
     else:
         return value
@@ -186,14 +187,18 @@ class SensorReading(object):
     """
 
     def __init__(self, reading, suffix):
+        self.broken_sensor_ids = {}
         self.health = const.Health.Ok
         self.type = reading['type']
         self.value = None
         self.imprecision = None
         self.states = []
+        self.state_ids = []
+        self.unavailable = 0
         try:
             self.health = reading['health']
             self.states = reading['states']
+            self.state_ids = reading['state_ids']
             self.value = reading['value']
             self.imprecision = reading['imprecision']
         except KeyError:
@@ -207,10 +212,12 @@ class SensorReading(object):
         return repr({
             'value': self.value,
             'states': self.states,
+            'state_ids': self.state_ids,
             'units': self.units,
             'imprecision': self.imprecision,
             'name': self.name,
             'type': self.type,
+            'unavailable': self.unavailable,
             'health': self.health
         })
 
@@ -247,6 +254,7 @@ class SDREntry(object):
     def __init__(self, entrybytes, ipmicmd, reportunsupported=False):
         # ignore record id for now, we only care about the sensor number for
         # moment
+        self.readable = True
         self.reportunsupported = reportunsupported
         self.ipmicmd = ipmicmd
         if entrybytes[2] != 0x51:
@@ -254,12 +262,14 @@ class SDREntry(object):
             raise NotImplementedError
         self.rectype = entrybytes[3]
         self.linearization = None
-        #most important to get going are 1, 2, and 11
+        # most important to get going are 1, 2, and 11
         self.sdrtype = TYPE_SENSOR  # assume a sensor
         if self.rectype == 1:  # full sdr
             self.full_decode(entrybytes[5:])
         elif self.rectype == 2:  # full sdr
             self.compact_decode(entrybytes[5:])
+        elif self.rectype == 3:  # event only
+            self.eventonly_decode(entrybytes[5:])
         elif self.rectype == 8:  # entity association
             self.association_decode(entrybytes[5:])
         elif self.rectype == 0x11:  # FRU locator
@@ -305,31 +315,48 @@ class SDREntry(object):
 
     def association_decode(self, entry):
         # table 43-4 Entity Associaition Record
-        #TODO(jbjohnso): actually represent this data
+        # TODO(jbjohnso): actually represent this data
         self.sdrtype = TYPE_UNKNOWN
 
+    def eventonly_decode(self, entry):
+        # table 43-3 event_only sensor record
+        self._common_decode(entry)
+        self.sensor_name = self.tlv_decode(entry[11], entry[12:])
+        self.readable = False
+
     def compact_decode(self, entry):
         # table 43-2 compact sensor record
         self._common_decode(entry)
         self.sensor_name = self.tlv_decode(entry[26], entry[27:])
 
+    def assert_trap_value(self, offset):
+        trapval = (self.sensor_type_number << 16) + (self.reading_type << 8)
+        return trapval + offset
+
     def _common_decode(self, entry):
-        # compact and full are very similar
+        # event only, compact and full are very similar
         # this function handles the common aspects of compact and full
         # offsets from spec, minus 6
         self.sensor_number = entry[2]
         self.entity = ipmiconst.entity_ids.get(
             entry[3], 'Unknown entity {0}'.format(entry[3]))
-        self.sensor_type_number = entry[7]
+        if self.rectype == 3:
+            self.sensor_type_number = entry[5]
+            self.reading_type = entry[6]  # table 42-1
+        else:
+            self.sensor_type_number = entry[7]
+            self.reading_type = entry[8]  # table 42-1
         try:
-            self.sensor_type = ipmiconst.sensor_type_codes[entry[7]]
+            self.sensor_type = ipmiconst.sensor_type_codes[
+                self.sensor_type_number]
         except KeyError:
-            self.sensor_type = "UNKNOWN type " + str(entry[7])
-        self.reading_type = entry[8]  # table 42-1
-            # 0: unspecified
-            # 1: generic threshold based
-            # 0x6f: discrete sensor-specific from table 42-3, sensor offsets
-            # all others per table 42-2, generic discrete
+            self.sensor_type = "UNKNOWN type " + str(self.sensor_type_number)
+        if self.rectype == 3:
+            return
+        # 0: unspecified
+        # 1: generic threshold based
+        # 0x6f: discrete sensor-specific from table 42-3, sensor offsets
+        # all others per table 42-2, generic discrete
         # numeric format is one of:
         # 0 - unsigned, 1 - 1s complement, 2 - 2s complement, 3 - ignore number
         # compact records are supposed to always write it as '3', presumably
@@ -360,8 +387,8 @@ class SDREntry(object):
             self.modunit
 
     def full_decode(self, entry):
-        #offsets are table from spec, minus 6
-        #TODO(jbjohnso): table 43-13, put in constants to interpret entry[3]
+        # offsets are table from spec, minus 6
+        # TODO(jbjohnso): table 43-13, put in constants to interpret entry[3]
         self._common_decode(entry)
         # now must extract the formula data to transform values
         # entry[18 to entry[24].
@@ -402,6 +429,7 @@ class SDREntry(object):
         output = {
             'name': self.sensor_name,
             'type': self.sensor_type,
+            'id': self.sensor_number,
         }
         if reading[1] & 0b100000:
             output['unavailable'] = 1
@@ -429,6 +457,7 @@ class SDREntry(object):
             upper = 'lower'
             lower = 'upper'
         output['states'] = []
+        output['state_ids'] = []
         output['health'] = const.Health.Ok
         if discrete:
             for state in range(8):
@@ -436,31 +465,40 @@ class SDREntry(object):
                     statedesc, health = self._decode_state(state)
                     output['health'] |= health
                     output['states'].append(statedesc)
+                    output['state_ids'].append(self.assert_trap_value(state))
             if len(reading) > 3:
                 for state in range(7):
                     if reading[3] & (0b1 << state):
                         statedesc, health = self._decode_state(state + 8)
                         output['health'] |= health
                         output['states'].append(statedesc)
+                        output['state_ids'].append(
+                            self.assert_trap_value(state + 8))
         else:
             if reading[2] & 0b1:
                 output['health'] |= const.Health.Warning
                 output['states'].append(lower + " non-critical threshold")
+                output['state_ids'].append(self.assert_trap_value(1))
             if reading[2] & 0b10:
                 output['health'] |= const.Health.Critical
                 output['states'].append(lower + " critical threshold")
+                output['state_ids'].append(self.assert_trap_value(2))
             if reading[2] & 0b100:
                 output['health'] |= const.Health.Failed
                 output['states'].append(lower + " non-recoverable threshold")
+                output['state_ids'].append(self.assert_trap_value(3))
             if reading[2] & 0b1000:
                 output['health'] |= const.Health.Warning
                 output['states'].append(upper + " non-critical threshold")
+                output['state_ids'].append(self.assert_trap_value(4))
             if reading[2] & 0b10000:
                 output['health'] |= const.Health.Critical
                 output['states'].append(upper + " critical threshold")
+                output['state_ids'].append(self.assert_trap_value(5))
             if reading[2] & 0b100000:
                 output['health'] |= const.Health.Failed
                 output['states'].append(upper + " non-recoverable threshold")
+                output['state_ids'].append(self.assert_trap_value(6))
         return SensorReading(output, self.unit_suffix)
 
     def _set_tmp_formula(self, value):
@@ -521,7 +559,7 @@ class SDREntry(object):
             (entry[4] & 0b11110000) << 2
         self.accuracyexp = (entry[4] & 0b1100) >> 2
         self.direction = entry[4] & 0b11
-            #0 = n/a, 1 = input, 2 = output
+        # 0 = n/a, 1 = input, 2 = output
         self.resultexponent = twos_complement((entry[5] & 0b11110000) >> 4, 4)
         bexponent = twos_complement(entry[5] & 0b1111, 4)
         # might as well do the math to 'b' now rather than wait for later
@@ -566,31 +604,40 @@ class SDR(object):
     :param ipmicmd: A Command class object
     """
     def __init__(self, ipmicmd):
-        self.ipmicmd = ipmicmd
+        self.ipmicmd = weakref.proxy(ipmicmd)
         self.sensors = {}
         self.fru = {}
         self.read_info()
 
     def read_info(self):
-        #first, we want to know the device id
-        rsp = self.ipmicmd.raw_command(netfn=6, command=1)
+        # first, we want to know the device id
+        rsp = self.ipmicmd.xraw_command(netfn=6, command=1)
+        rsp['data'] = bytearray(rsp['data'])
         self.device_id = rsp['data'][0]
         self.device_rev = rsp['data'][1] & 0b111
         # Going to ignore device available until get sdr command
         # since that provides usefully distinct state and this does not
         self.fw_major = rsp['data'][2] & 0b1111111
         self.fw_minor = "%02X" % rsp['data'][3]  # BCD encoding, oddly enough
-        if rsp['data'][1] & 0b10000000:
-            # For lack of any system with 'device sdrs', raise an
-            # exception when they are encountered for now, implement or
-            # ignore later
-            raise NotImplementedError
         self.ipmiversion = rsp['data'][4]  # 51h = 1.5, 02h = 2.0
         self.mfg_id = rsp['data'][8] << 16 + rsp['data'][7] << 8 + \
             rsp['data'][6]
         self.prod_id = rsp['data'][10] << 8 + rsp['data'][9]
         if len(rsp['data']) > 11:
             self.aux_fw = self.decode_aux(rsp['data'][11:15])
+        if rsp['data'][1] & 0b10000000 and rsp['data'][5] & 0b10 == 0:
+            # The device has device sdrs, also does not support SDR repository
+            # device, so we are meant to use an alternative mechanism to get
+            # SDR data
+            if rsp['data'][5] & 1:
+                # The device has sensor device support, so in theory we should
+                # be able to proceed
+                # However at the moment, we haven't done so
+                raise NotImplementedError
+            return
+            # We have Device SDR, without SDR Repository device, but
+            # also without sensor device support, no idea how to
+            # continue
         self.get_sdr()
 
     def get_sdr_reservation(self):
@@ -600,19 +647,20 @@ class SDR(object):
         return rsp['data'][0] + (rsp['data'][1] << 8)
 
     def get_sdr(self):
-        repinfo = self.ipmicmd.raw_command(netfn=0x0a, command=0x20)
+        repinfo = self.ipmicmd.xraw_command(netfn=0x0a, command=0x20)
+        repinfo['data'] = bytearray(repinfo['data'])
         if (repinfo['data'][0] != 0x51):
             # we only understand SDR version 51h, the only version defined
             # at time of this writing
             raise NotImplementedError
-        #NOTE(jbjohnso): we actually don't need to care about 'numrecords'
+        # NOTE(jbjohnso): we actually don't need to care about 'numrecords'
         # since FFFF marks the end explicitly
-        #numrecords = (rsp['data'][2] << 8) + rsp['data'][1]
-        #NOTE(jbjohnso): don't care about 'free space' at the moment
-        #NOTE(jbjohnso): most recent timstamp data for add and erase could be
+        # numrecords = (rsp['data'][2] << 8) + rsp['data'][1]
+        # NOTE(jbjohnso): don't care about 'free space' at the moment
+        # NOTE(jbjohnso): most recent timstamp data for add and erase could be
         # handy to detect cache staleness, but for now will assume invariant
         # over life of session
-        #NOTE(jbjohnso): not looking to support the various options in op
+        # NOTE(jbjohnso): not looking to support the various options in op
         # support, ignore those for now, reservation if some BMCs can't read
         # full SDR in one slurp
         recid = 0
@@ -620,6 +668,7 @@ class SDR(object):
         offset = 0
         size = 0xff
         chunksize = 128
+        self.broken_sensor_ids = {}
         while recid != 0xffff:  # per 33.12 Get SDR command, 0xffff marks end
             newrecid = 0
             currlen = 0
@@ -670,21 +719,30 @@ class SDR(object):
             if newrecid == recid:
                 raise exc.BmcErrorException("Incorrect SDR record id from BMC")
             recid = newrecid
+        for sid in self.broken_sensor_ids:
+            try:
+                del self.sensors[sid]
+            except KeyError:
+                pass
 
     def get_sensor_numbers(self):
-        return self.sensors.iterkeys()
+        for number in self.sensors:
+            if self.sensors[number].readable:
+                yield number
 
     def add_sdr(self, sdrbytes):
         newent = SDREntry(sdrbytes, self.ipmicmd)
         if newent.sdrtype == TYPE_SENSOR:
             id = newent.sensor_number
             if id in self.sensors:
-                raise exc.BmcErrorException("Duplicate sensor number %d" % id)
+                self.broken_sensor_ids[id] = True
+                return
             self.sensors[id] = newent
         elif newent.sdrtype == TYPE_FRU:
             id = newent.fru_number
             if id in self.fru:
-                raise exc.BmcErrorException("Duplicate FRU identifier %d" % id)
+                self.broken_sensor_ids[id] = True
+                return
             self.fru[id] = newent
 
     def decode_aux(self, auxdata):
@@ -707,4 +765,4 @@ if __name__ == "__main__":  # test code
             continue
         reading = sdr.sensors[number].decode_sensor_reading(rsp['data'])
         if reading is not None:
-            print repr(reading)
+            print(repr(reading))
